/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 
         International, see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.
--------------------------------------------------------------------------------

This macro provides an example how to set up an RGNG grid cell simulation.

Common usage in the ROOT command line would be, e.g.:

root [0] .x tgng_load.C
root [1] .L macros/grid_inputs.C+
root [2] .L macros/grid_example.C
root [3] run("example.root",2000000)

*/





/*
	This function setups and starts an RGNG experiment. The results are stored
	in "logfile". The RGNG will be fed with "sampleCnt" random inputs.
*/
void run(const char *logfile, int sampleCnt)
{
// experiment setup
	LongTestParameters params;

	// set up the TL and BL learning rates
	params.tlPrimaryRate      = 0.04; 
	params.tlSecondaryRate    = 0.04; 

	params.blPrimaryRate      = 0.01; 
	params.blSecondaryRate    = 0.0001; 

	// set up the number of TL and BL units
	params.topLevelNodes    = 100;
	params.bottomLevelNodes = 20;

	// ramping and learning inhibition not used
	params.tlPrimaryRateEnd   = 0.0005; 
	params.tlSecondaryRateEnd = 0.0001; 
	params.tlRamp             = 500000; 
    params.tlGamma            = 0.02; 
	params.tlUseRamp          = kFALSE; 
	params.tlUseLearnInhib    = kFALSE;

	params.tlLambda           = 1000;
	params.tlTmax             = 300;

	// ramping and learning inhibition not used
	params.blPrimaryRateEnd   = 0.0005; 
	params.blSecondaryRateEnd = 0.0001; 
	params.blRamp             = 500000; 
    params.blGamma            = 0.2; 
	params.blUseRamp          = kFALSE; 
	params.blUseLearnInhib    = kFALSE;

	params.blLambda           = 1000;
	params.blTmax             = 300;

	// set epsilon_r for TL and BL (not distinguished here)
    params.nbAttenuation = 0.01;

    params.minkowskiParam = 2;
    params.recMode = TGNGMinkowskiDistance::rmFeed;

    // set logging interval for the whole GNG. Inputs, Activations, Errors, etc.
    // are always logged at an interval of 1.
    params.logInterval = 10000;

    // setting up input dimension and "fade"-parameter (see feedRandom())
    params.inputDim = 100;
    params.inpFade  = 8;
    params.inpResample = 8; // set up resampling to avoid aliasing effects


    printf("Running experiment with:\n");
    printf("topLevelNodes:      %d\n",params.topLevelNodes);
    printf("bottomLevelNodes:   %d\n\n",params.bottomLevelNodes);
    printf("tlPrimaryRate:      %f\n",params.tlPrimaryRate);
    printf("tlPrimaryRateEnd:   %f\n",params.tlPrimaryRateEnd);
    printf("tlSecondaryRate:    %f\n",params.tlSecondaryRate);
    printf("tlSecondaryRateEnd: %f\n",params.tlSecondaryRateEnd);
    printf("tlRamp:             %f\n",params.tlRamp);
    printf("tlGamma:            %f\n",params.tlGamma);
    printf("tlTmax:             %d\n",params.tlTmax);
    printf("tlLambda:           %d\n",params.tlLambda);
    printf("tlUseRamp:          %d\n",(int)params.tlUseRamp);
    printf("tlUseLearnInhib:    %d\n\n",(int)params.tlUseLearnInhib);
    printf("blPrimaryRate:      %f\n",params.blPrimaryRate);
    printf("blPrimaryRateEnd:   %f\n",params.blPrimaryRateEnd);
    printf("blSecondaryRate:    %f\n",params.blSecondaryRate);
    printf("blSecondaryRateEnd: %f\n",params.blSecondaryRateEnd);
    printf("blRamp:             %f\n",params.blRamp);
    printf("blGamma:            %f\n",params.blGamma);
    printf("blLambda:           %d\n",params.blLambda);
    printf("blTmax:             %d\n",params.blTmax);
    printf("nbAttenuation:      %f\n",params.nbAttenuation);
    printf("blUseRamp:          %d\n",(int)params.blUseRamp);
    printf("blUseLearnInhib:    %d\n\n",(int)params.blUseLearnInhib);
    printf("minkowskiParam:     %f\n",params.minkowskiParam);
    printf("recMode:            TGNGMinkowskiDistance::rmFeed\n\n");
    printf("logInterval:        %d\n",params.logInterval);
    printf("inputDim:           %d\n",params.inputDim);
    printf("inpFade:            %d\n",params.inpFade);
    printf("inpResampling:      %d\n\n",params.inpResample);
    printf("logfile:            %s\n",logfile);

    LongTest(params,logfile, sampleCnt);

    printf("--------------------------------------\n");

}




