#ifndef TGNGEXTENSION_H
#define TGNGEXTENSION_H

/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the basic interface for gng extensions. During the operation
of the gng, the different callback methods defined below will be called if an
extension is registered to the gng.
*/

#include <TObject.h>

class TGNG;
class TGNGUnit;
class TGNGEdge;
class TGNGDataElement;

class TGNGExtension : public TObject 
{
	ClassDef(TGNGExtension,2);

	// v2: added second bmu/dist to feed and peek

public:

	TGNGExtension();

	virtual void OnGNGInit(TGNG *gng);
	virtual void OnGNGReset(TGNG *gng);

	virtual void OnError(TGNG *gng, UInt_t errorCode, const char *errorMsg);

	virtual void OnPeekResult(TGNG                  *gng, 
		                      TGNGUnit              *bmu, 
		                      Double_t               bmuDist,
		                      TGNGUnit              *bmu2,
		                      Double_t               bmu2Dist, 
		                      const TGNGDataElement *input);
	
	virtual void OnFeedResult(TGNG                  *gng, 
		                      TGNGUnit              *bmu, 
		                      Double_t               bmuDist,
		                      TGNGUnit              *bmu2,
		                      Double_t               bmu2Dist, 
		                      const TGNGDataElement *input);

	virtual void OnNewUnit(TGNG *gng, TGNGUnit *newUnit);	
	virtual void OnUnitRemoval(TGNG *gng, TGNGUnit *oldUnit);	

	virtual void OnNewEdge(TGNG *gng, TGNGEdge *newEdge);	
	virtual void OnEdgeRemoval(TGNG *gng, TGNGEdge *newEdge);

	virtual void OnUnitUpdate(TGNG                  *gng, 
		                      TGNGUnit              *bmu,
		                      Double_t               bmuDist,
		                      TGNGUnit              *bmu2,
		                      Double_t               bmu2Dist,
		                      TGNGEdge              *bmuEdge,
		                      const TGNGDataElement *input    );

	virtual void OnLambda(TGNG *gng, ULong64_t inputCnt);
};



#endif


