/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the definition of an approximate earth movers distance 
measure that can be used with the gng.

*/
#include <TMath.h>

#include "../tools/EMD.h"
#include "../datatypes/TGNGVectorData.h"
#include "../core/TGNG.h"

#include "TGNGApproxEMD.h"


TGNGApproxEMD::TGNGApproxEMD() : 
	TGNGDistanceMeasure(),
	fNormalize(kTRUE)
{}

TGNGApproxEMD::TGNGApproxEMD(Bool_t normalize) : 
	TGNGDistanceMeasure(),
	fNormalize(normalize)
{}

Double_t TGNGApproxEMD::GetDistance(const TGNGDataElement *a, 
                        	        const TGNGDataElement *b, TGNG *gng)
{
	const TGNGVectorData *aVecData = dynamic_cast<const TGNGVectorData*>(a);
	const TGNGVectorData *bVecData = dynamic_cast<const TGNGVectorData*>(b);
	if ((aVecData != 0) && (bVecData != 0)) {
		return EMD::ApproxEMD(&(const_cast<TGNGVectorData*>(aVecData)->fVector),
			                  &(const_cast<TGNGVectorData*>(bVecData)->fVector),fNormalize);
	}

	gng->RaiseError(TGNG::ecExtension,"No compatible TGNGDataElement provided to GetDistance()!");
	return TMath::QuietNaN();
}

Bool_t TGNGApproxEMD::IsCompatible(const TGNGDataElement *a, TGNG *gng)
{
	Bool_t result = kFALSE;

	const TGNGVectorData *vData = dynamic_cast<const TGNGVectorData*>(a);
	if (vData != 0) {
		result = kTRUE;
		Int_t vecSize = vData->fVector.GetNrows();
		if ((vecSize & (vecSize - 1)) != 0) {
			gng->RaiseError(TGNG::ecExtension,"Vector size must be power of 2!");
			result = kFALSE;
		}
		return result;		
	}

	return result;
}

void TGNGApproxEMD::SetNormalize(Bool_t normalize)
{
	fNormalize = normalize;
}


Bool_t TGNGApproxEMD::GetNormalize()
{
	return fNormalize;
}


