/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the definition of a hierarchical clustering node used by
TGNGHierarchicalClusterExt.
*/

#include "../../core/TGNGUnit.h"

#include "TGNGHcNode.h"

TGNGHcNode::TGNGHcNode() : 
	TObject(),
	fLeafList(),
	fParent(0),
	fChildA(0),
	fChildB(0),
	fLink(0),
	fDepth(0)
{
	fLeafList.SetOwner(kFALSE);
}

TGNGHcNode::TGNGHcNode(TGNGUnit *leaf) : 
	TObject(),
	fLeafList(),
	fParent(0),
	fChildA(0),
	fChildB(0),
	fLink(0),
	fDepth(0)
{
	fLeafList.SetOwner(kFALSE);
	fLeafList.Add(leaf);
}

TGNGHcNode::TGNGHcNode(TGNGHcNode *childA, TGNGHcNode *childB) : 
	TObject(),
	fLeafList(),
	fParent(0),
	fChildA(childA),
	fChildB(childB),
	fLink(0),
	fDepth(0)
{
	childA->fParent = this;
	childB->fParent = this;
	fLeafList.SetOwner(kFALSE);
	fLeafList.AddAll(&(childA->fLeafList));
	fLeafList.AddAll(&(childB->fLeafList));
	fDepth = ((childA->fDepth > childB->fDepth) ? childA->fDepth : childB->fDepth) + 1;
}



TGNGHcNode::~TGNGHcNode()
{	
	if (fChildA != 0)
		fChildA->fParent = 0;

	if (fChildB != 0)
		fChildB->fParent = 0;

	if (fParent != 0) {
		if (fParent->fChildA == this)
			fParent->fChildA = 0;
		if (fParent->fChildB == this)
			fParent->fChildB = 0;		

		if ((fParent->fChildA == 0) && (fParent->fChildB == 0)) {
			delete fParent;
		} else {
			//fParent->fLeafList.RemoveAll(&fLeafList);
		}
	}

}


