/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the definition of an extension that assigns an overall activity
to the gng.
*/
#include <TMath.h>

#include "../../core/TGNGUnit.h"
#include "../../core/TGNG.h"

#include "TGNGActivityExt.h"

TGNGActivityExt::TGNGActivityExt() :
	TGNGExtension(),
	fIdentifier("ACTIVITY")	
{}

void TGNGActivityExt::OnGNGInit(TGNG *gng)
{
	if (gng->fAttributes.FindObject(&fIdentifier) != 0) {
		gng->RaiseError(TGNG::ecExtension,"GNG has already an attribute \"ACTIVITY\"!");
		return;
	}
	gng->fAttributes.Add(new TObjString(fIdentifier),new TGNGActivity());

}

void TGNGActivityExt::OnPeekResult(TGNG                  *gng, 
                                   TGNGUnit              *bmu, 
                  				   Double_t               bmuDist,
                  				   TGNGUnit              *bmu2,
                  				   Double_t               bmu2Dist, 
                  				   const TGNGDataElement *)
{
	NewInput(gng,bmu,bmuDist,bmu2,bmu2Dist);
}

void TGNGActivityExt::OnFeedResult(TGNG                  *gng, 
                  				   TGNGUnit              *bmu, 
                  				   Double_t               bmuDist,
                  				   TGNGUnit              *bmu2,
                  				   Double_t               bmu2Dist, 
                  				   const TGNGDataElement *)
{
	NewInput(gng,bmu,bmuDist,bmu2,bmu2Dist);
}

void TGNGActivityExt::NewInput(TGNG *gng, TGNGUnit *u1, Double_t d1, TGNGUnit *u2, Double_t d2)
{
	TObject *attrib = gng->fAttributes.GetValue(&fIdentifier);

	if (attrib == 0) {
		return;
	}

	TGNGActivity *act = static_cast<TGNGActivity*>(attrib);
	if (act == 0) {
		gng->RaiseError(TGNG::ecExtension,"GNG has no attribute \"ACTIVITY\"!");
		return;		
	}
	act->d1 = d1;
	act->d2 = d2;

	Double_t ratio = 1.0;

	TGNGDistanceMeasure *dm = gng->GetDistanceMeasure();

	Double_t relDist = dm->GetDistance(u1->fPrototype,u2->fPrototype,gng);

	if ((TMath::IsNaN(relDist) == kFALSE) && (relDist != 0)) {
		ratio = ((d1 - d2) / relDist) + 1.0;
//		printf("rd: %f / d1: %f / d2: %f\n",relDist,d1,d2);
	}

	if (ratio > 1.0) ratio = 1.0;
	if (ratio < 0.0) ratio = 0.0;

	act->a1 = 1.0 - ratio;


}

