/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the definiton of a helper extension that allows to feed
the local input space histograms within a gng as data-vectors into a second
gng, i.e., the second gng is used to quantize the local input space histograms
of the first gng.

This extension must be registered with the first gng and should get the
second gng as a parameter to its constructor. Alternatively, the second gng
can be set by SetReceiver().

*/

#include <TRandom.h>

#include "../../core/TGNGEdge.h"
#include "../../core/TGNGUnit.h"
#include "../../core/TGNG.h"

#include "../../datatypes/TGNGVectorData.h"

#include "TGNGLishFeedExt.h"



TGNGLishFeedExt::TGNGLishFeedExt() : 
	TGNGExtension(),
	fReceiver(0),
	lishKey("LISH"),
	fDelay(0),
	fInpCnt(0)
{}

TGNGLishFeedExt::TGNGLishFeedExt(TGNG *receiver, Int_t delay) : 
	TGNGExtension(),
	fReceiver(receiver),
	lishKey("LISH"),
	fDelay(delay),
	fInpCnt(0)
{}

void TGNGLishFeedExt::OnFeedResult(TGNG*, TGNGUnit *bmu, Double_t,
								   TGNGUnit*, Double_t, const TGNGDataElement*)
{
	if ((fReceiver != 0) && (++fInpCnt > fDelay)) {
		Int_t randomEdge = gRandom->Integer(bmu->fEdges.GetEntriesFast());
		TGNGEdge *edge = static_cast<TGNGEdge*>(bmu->fEdges.At(randomEdge));

		// get lish
		TH1D *lish = static_cast<TH1D*>(edge->fAttributes.GetValue(&lishKey));
		if (lish == 0) {
			return;
		}

		Int_t lishSize = lish->GetNbinsX();
		TGNGVectorData data(lishSize);
		for (Int_t i = 0; i < lishSize; ++i)
			data.fVector[i] = lish->GetBinContent(i+1);

		fReceiver->Feed(&data);
	}
}

void TGNGLishFeedExt::SetReceiver(TGNG *receiver, Int_t delay)
{
	fReceiver = receiver;
	fDelay    = delay;
	fInpCnt   = 0;
}

void TGNGLishFeedExt::OnGNGReset(TGNG*)
{
	fInpCnt = 0;	
}