/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the definition of a statistic extension that collects 
information about the best matching and second best matching distances that
occur across all inputs to the gng.

The resulting statistics can be accessed via the GetBMUDist() and GetBMU2Dist()
methods.
*/

#include "../../core/TGNGEdge.h"
#include "../../core/TGNGUnit.h"
#include "../../core/TGNG.h"
#include "../../core/TGNGDataElement.h"

#include "TGNGDistStat.h"


TGNGDistStat::TGNGDistStat() : 
	TGNGExtension(),
	fBmuDists(0),
	fBmu2Dists(0),
	fLow(0),
	fHigh(1),
	fBinCount(1000)
{
	// default constructor setting the bin count to 1000 and the distance range
	// between 0 and 1

	fBmuDists = new TH1D("BMUdist","BMU distance distribution",fBinCount,fLow,fHigh);
	fBmu2Dists = new TH1D("BMU2dist","second BMU distance distribution",fBinCount,fLow,fHigh);

	fBmuDists->SetDirectory(0);
	fBmu2Dists->SetDirectory(0);
}

TGNGDistStat::TGNGDistStat(Int_t bincnt, Double_t low, Double_t high) : 
	TGNGExtension(),
	fBmuDists(0),
	fBmu2Dists(0),
	fLow(low),
	fHigh(high),
	fBinCount(bincnt)
{
	// constructor setting the bin count to "bincnt" and the distance range
	// between "low" and "high"

	fBmuDists = new TH1D("BMUdist","BMU distance distribution",fBinCount,fLow,fHigh);
	fBmu2Dists = new TH1D("BMU2dist","second BMU distance distribution",fBinCount,fLow,fHigh);

	fBmuDists->SetDirectory(0);
	fBmu2Dists->SetDirectory(0);
}

TGNGDistStat::~TGNGDistStat()
{
	delete fBmuDists;
	delete fBmu2Dists;	
}

void TGNGDistStat::OnGNGReset(TGNG*) 
{
	delete fBmuDists;
	delete fBmu2Dists;

	fBmuDists = new TH1D("BMUdist","BMU distance distribution",fBinCount,fLow,fHigh);
	fBmu2Dists = new TH1D("BMU2dist","second BMU distance distribution",fBinCount,fLow,fHigh);

	fBmuDists->SetDirectory(0);
	fBmu2Dists->SetDirectory(0);

}

void TGNGDistStat::OnUnitUpdate(TGNG*, TGNGUnit*, Double_t bmuDist, TGNGUnit*,
	                      		Double_t bmu2Dist, TGNGEdge*, const TGNGDataElement*) 
{
	fBmuDists->Fill(bmuDist);
	fBmu2Dists->Fill(bmu2Dist);
}

TH1D* TGNGDistStat::GetBMUDist()
{
	TH1D *result = (TH1D*)fBmuDists->Clone();
	result->SetDirectory(0);
	return result;
}

TH1D* TGNGDistStat::GetBMU2Dist()
{
	TH1D *result = (TH1D*)fBmu2Dists->Clone();
	result->SetDirectory(0);
	return result;
}

void TGNGDistStat::SetHistParams(Int_t bincnt, Double_t low, Double_t high)
{
	fLow      = low;
	fHigh     = high;
	fBinCount = bincnt;

	delete fBmuDists;
	delete fBmu2Dists;

	fBmuDists = new TH1D("BMUdist","BMU distance distribution",fBinCount,fLow,fHigh);
	fBmu2Dists = new TH1D("BMU2dist","second BMU distance distribution",fBinCount,fLow,fHigh);

	fBmuDists->SetDirectory(0);
	fBmu2Dists->SetDirectory(0);	
}
