#ifndef TGNGDISTSTAT_H
#define TGNGDISTSTAT_H

/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the declaration of a statistic extension that collects 
information about the best matching and second best matching distances that
occur across all inputs to the gng.

The resulting statistics can be accessed via the GetBMUDist() and GetBMU2Dist()
methods.
*/


#include <TObject.h>
#include <TH1D.h>

#include "../../core/TGNGExtension.h"

class TGNGDistStat : public TGNGExtension 
{
	ClassDef(TGNGDistStat,2);

public:

	TGNGDistStat();
	TGNGDistStat(Int_t bincnt, Double_t low, Double_t high);
	~TGNGDistStat();

	void OnGNGReset(TGNG *gng);

	void OnUnitUpdate(TGNG                  *gng, 
	                  TGNGUnit              *bmu,
	                  Double_t               bmuDist,
	                  TGNGUnit              *bmu2,
	                  Double_t               bmu2Dist,
	                  TGNGEdge              *bmuEdge,
		              const TGNGDataElement *input    );

	TH1D* GetBMUDist();
	TH1D* GetBMU2Dist();

	void SetHistParams(Int_t bincnt, Double_t low, Double_t high);

private:

	TH1D *fBmuDists;   //|| histogram of first distances
	TH1D *fBmu2Dists;  //|| histogram of second distances

	Double_t fLow;      // lower bound of range
	Double_t fHigh;     // higher bound of range (exclusive)
	Int_t    fBinCount; // number of bins

};



#endif


