#ifndef TGNGFULLLOGGEREXT_H
#define TGNGFULLLOGGEREXT_H

/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the declaration of a logging extension that stores not only
the gng at every time step but also every event that takes place, e.g., unit
addition / removal, edge addition / removal, etc.
*/


#include "../../core/TGNGExtension.h"

class TGNG;

class TGNGFullLoggerExt : public TGNGExtension
{
	ClassDef(TGNGFullLoggerExt,3);

	// v2: reflect changes in TGNGExtension
	// v3: added GetCurrentLogFile() method
	// v4: added raw input support

public:

	TGNGFullLoggerExt();
	explicit TGNGFullLoggerExt(const char *file);
	~TGNGFullLoggerExt();

	void SetLogFile(const char *file);
	void CloseLogFile();

	void OnError(TGNG *gng, UInt_t errorCode, const char *errorMsg);

	void OnGNGInit(TGNG *gng);
	void OnGNGReset(TGNG *gng);

	void OnPeekResult(TGNG *gng, TGNGUnit *bmu,  Double_t bmuDist, 
		                         TGNGUnit *bmu2, Double_t bmu2Dist, const TGNGDataElement *input);
	void OnFeedResult(TGNG *gng, TGNGUnit *bmu,  Double_t bmuDist, 
		                         TGNGUnit *bmu2, Double_t bmu2Dist, const TGNGDataElement *input);

	void OnNewUnit(TGNG *gng, TGNGUnit *newUnit);	
	void OnUnitRemoval(TGNG *gng, TGNGUnit *oldUnit);	

	void OnNewEdge(TGNG *gng, TGNGEdge *newEdge);	
	void OnEdgeRemoval(TGNG *gng, TGNGEdge *newEdge);

	void OnUnitUpdate(TGNG                  *gng, 
	                  TGNGUnit              *bmu,
	                  Double_t               bmuDist,
	                  TGNGUnit              *bmu2,
	                  Double_t               bmu2Dist,
	                  TGNGEdge              *bmuEdge,
	                  const TGNGDataElement *input    );

	void SetGNGLogInterval(UInt_t val);
	UInt_t GetGNGLogInterval();

	void SetGNGLogEnabled(Bool_t enable);
	void SetErrorLogEnabled(Bool_t enable);
	void SetPeekLogEnabled(Bool_t enable);
	void SetFeedLogEnabled(Bool_t enable);
	void SetEdgeLogEnabled(Bool_t enable);
	void SetUnitLogEnabled(Bool_t enable);
	void SetUpdateLogEnabled(Bool_t enable);

	Bool_t GetGNGLogEnabled();
	Bool_t GetErrorLogEnabled();
	Bool_t GetPeekLogEnabled();
	Bool_t GetFeedLogEnabled();
	Bool_t GetEdgeLogEnabled();
	Bool_t GetUnitLogEnabled();
	Bool_t GetUpdateLogEnabled();

	TFile *GetCurrentLogFile();

private:
	TFile *fLogFile;    //! current log file
	TTree *fGngTree;    //! current gng log tree
	TTree *fErrorTree;  //! current error log tree
	TTree *fPeekTree;   //! current peek log tree
	TTree *fFeedTree;   //! current feed log tree
	TTree *fEdgeTree;   //! current edge tree
	TTree *fUnitTree;   //! current unit tree
	TTree *fUpdateTree; //! current update tree

	Int_t fNextTreeIdx; // next tree idx to use

	ULong64_t        fInpCnt;    //! storage for current gng input cnt
	TGNG            *fGNG;       //! storage for current gng
	Int_t            fBMUIdx;    //! storage for a bmu idx
	Int_t            fBMU2Idx;   //! storage for a bmu idx
	Double_t         fBMUDist;   //! storage for a bmu distance
	Double_t         fBMU2Dist;  //! storage for a bmu distance
	TGNGDataElement *fInput;     //! storage for current input
	Int_t            fEdgeIdx;   //! storage for an edge idx
	Int_t            fEventType; //! storage for an event type

	TString *fErrorMsg;  //! storage for error message
	UInt_t   fErrorCode; //! storage for error code
	UInt_t   fErrorCnt;  //! storage for error counter

	UInt_t fGNGLogInterval; // interval for snapshots of the whole gng

	Bool_t fGNGLogEnabled; // indicates wether whole gng logging is enabled
	Bool_t fErrorLogEnabled; // indicates wether error logging is enabled
	Bool_t fPeekLogEnabled; // indicates wether peek logging is enabled
	Bool_t fFeedLogEnabled; // indicates wether feed logging is enabled
	Bool_t fEdgeLogEnabled; // indicates wether edge logging is enabled
	Bool_t fUnitLogEnabled; // indicates wether unit logging is enabled
	Bool_t fUpdateLogEnabled; // indicates wether update logging is enabled

	TObject fRawInputDummy; // dummy object stored when no raw input was available
	TObject *fRawInput; //! storage for raw input
};



#endif


