#ifndef TGNGUNITLOGGEREXT_H
#define TGNGUNITLOGGEREXT_H

/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the declaration of a logging mechanism that allows to save
the whole gng network into a root file at regular intervals during learning.
*/

#include "../../core/TGNGExtension.h"

class TObjArray;

class TGNGUnitLoggerExt : public TGNGExtension
{
	ClassDef(TGNGUnitLoggerExt,2);

	// v2: reflect changes of TGNGExtension

public:

	TGNGUnitLoggerExt();
	explicit TGNGUnitLoggerExt(const char *file);
	~TGNGUnitLoggerExt();

	void SetLogFile(const char *file);
	void CloseLogFile();

	void  SetLogInterval(Int_t step);
	Int_t GetLogInterval();

	void OnGNGInit(TGNG *gng);
	void OnGNGReset(TGNG *gng);

	void OnPeekResult(TGNG *gng, TGNGUnit *bmu, Double_t bmuDist, 
		              TGNGUnit *bmu2, Double_t bmu2Dist, const TGNGDataElement *input);

	void OnFeedResult(TGNG *gng, TGNGUnit *bmu, Double_t bmuDist,
	                  TGNGUnit *bmu2, Double_t bmu2Dist, const TGNGDataElement *input);

private:
	void LogUnits(TGNG *gng);

	TFile *fLogFile; //! current log file
	TTree *fLogTree; //! current log tree

	Int_t fNextTreeIdx; // next tree idx to use

	Int_t fLogInterval; // specifies how often a snapshot of all units is taken

	ULong64_t  fInpCnt; //! storage for current gng input cnt
	TObjArray *fUnits; //! storage for current gng units
};



#endif


