#ifndef TGNGMATRIXVIS_H
#define TGNGMATRIXVIS_H

/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the declaration of a gng visualization that plots the gng
units in an incidence matrix. The matrix can be colored by distance, can 
indicate edges between units, and can draw a small miniature local input space
histogram for each edge.
*/

#include <vector>

#include <TVirtualPad.h>
#include <TTimeStamp.h>
#include <TAttLine.h>
#include <TAttMarker.h>
#include <TAttText.h>
#include <TAttFill.h>
#include <TObjString.h>

#include "../../core/TGNGExtension.h"

class TGNGMatrixVis : public TGNGExtension, public TAttLine, public TAttMarker, public TAttText, public TAttFill
{
	ClassDef(TGNGMatrixVis,2);

	// v2: reflecting changes in TGNGExtension

public:

	TGNGMatrixVis();

	void Draw(Option_t* option = "");
	void Paint(Option_t *option);

	// virtual methods
	void OnPeekResult(TGNG *gng, TGNGUnit *bmu, Double_t bmuDist,
					  TGNGUnit *bmu2, Double_t bmu2Dist, const TGNGDataElement *input);
	void OnFeedResult(TGNG *gng, TGNGUnit *bmu, Double_t bmuDist,
					  TGNGUnit *bmu2, Double_t bmu2Dist, const TGNGDataElement *input);

	void SetUpdateInterval(Double_t sec);

	void SetRange(Int_t minX = -1, Int_t maxX = -1, Int_t minY = -1, Int_t maxY = -1);

	void SetEdgeColor(Color_t color);
	void SetEdgeStyle(Style_t style);
	void SetEdgeWidth(Width_t width);

	void SetAdjMarkerColor(Color_t color);
	void SetAdjMarkerSize(Size_t	size);
	void SetAdjMarkerStyle(Style_t style);

	void SetTableTextTopOffsetX(Double_t ox);
	void SetTableTextTopOffsetY(Double_t oy);
	void SetTableTextLeftOffsetX(Double_t ox);
	void SetTableTextLeftOffsetY(Double_t oy);
	void SetTableTextColor(Color_t color);
	void SetTableTextFont(Font_t font);
	void SetTableTextSize(Float_t size);

	void SetLishBgColor(Color_t   color, Float_t alpha);
	void SetLishBgStyle(Style_t   style);
	void SetLishLineColor(Color_t color);
	void SetLishErrorLineColor(Color_t color);
	void SetLishLineWidth(Width_t width);
	void SetLishLineStyle(Style_t style);

	void SetPalette(Int_t a);

private:
	void UpdateVis();

	TVirtualPad *fCurrentPad; //! stores current pad we are drawn to

	Double_t   fUpdateInterval; // update interval
	TTimeStamp fLastUpdate;     // last time the view was painted

	TGNG     *fCurrentGNG; //! stores current gng provided OnFeedResult/OnPeekResult
	TGNGUnit *fCurrentBMU; //! stores current unit provided OnFeedResult/OnPeekResult

	Color_t	fEdgeColor; // color used for edges
	Style_t	fEdgeStyle; // style used for edges
	Width_t	fEdgeWidth; // width used for edges

	Color_t	fAdjMarkerColor; // color used for units
	Size_t	fAdjMarkerSize;  // size used for units
	Style_t	fAdjMarkerStyle;	// marker style used for units

	Double_t fTableTextTopOffsetX; // unit text x offset from center
	Double_t fTableTextTopOffsetY; // unit text y offset from center
	Double_t fTableTextLeftOffsetX; // unit text x offset from center
	Double_t fTableTextLeftOffsetY; // unit text y offset from center
	Color_t  fTableTextColor;   // unit text color
	Font_t	 fTableTextFont;    // unit text font
	Float_t	 fTableTextSize;    // unit text size

	Int_t fMinX; // minimum x range
	Int_t fMaxX; // maximum x range
	Int_t fMinY; // minimum y range
	Int_t fMaxY; // maximum y range

	Color_t	fLishBgColor; // color used for lish background
	Float_t fLishBgColorAlpha; // alpha for lish background
	Style_t	fLishBgStyle; // marker style used for lish background
	Color_t	fLishLineColor; // color used for lish histogram lines
	Color_t	fLishErrorLineColor; // color used for lish histogram lines
	Width_t fLishLineWidth; // width for lish histogram lines
	Style_t fLishLineStyle; // style for lish histogram lines

	Int_t fPaletteA; // id of first palette

	std::vector<Color_t> fPalA; //! colors of first palette

};



#endif



