#ifndef TGNGVECTOR2DVIS_H
#define TGNGVECTOR2DVIS_H

/*
FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
License: Creative Commons Attribution-Noncommercial-Share Alike 4.0 International, 
         see README file that comes with this work
Author: J. Kerdels

The use of this code for military and/or intelligence purposes is disapproved of.

This file contains the declaration of a simple GNG network visualization if
the GNG contains only two-dimensional input vectors of type TGNGVectorData
*/

#include <TVirtualPad.h>
#include <TTimeStamp.h>
#include <TAttLine.h>
#include <TAttMarker.h>
#include <TAttText.h>
#include <TAttFill.h>

#include "../../core/TGNGExtension.h"

class TGNGVector2DVis : public TGNGExtension, public TAttLine, public TAttMarker, public TAttText, public TAttFill
{
	ClassDef(TGNGVector2DVis,2);

	// v2: reflecting changes in TGNGExtension

public:

	TGNGVector2DVis();

	void Draw(Option_t* option = "");
	void Paint(Option_t *option);

	// virtual methods
	void OnPeekResult(TGNG *gng, TGNGUnit *bmu, Double_t bmuDist, 
					  TGNGUnit *bmu2, Double_t bmu2Dist, const TGNGDataElement *input);
	void OnFeedResult(TGNG *gng, TGNGUnit *bmu, Double_t bmuDist,
					  TGNGUnit *bmu2, Double_t bmu2Dist, const TGNGDataElement *input);

	void SetUpdateInterval(Double_t sec);

	void SetEdgeColor(Color_t color);
	void SetEdgeStyle(Style_t style);
	void SetEdgeWidth(Width_t width);

	void SetUnitColor(Color_t color);
	void SetUnitSize(Size_t	size);
	void SetUnitStyle(Style_t style);

	void SetUnitTextOffsetX(Double_t ox);
	void SetUnitTextOffsetY(Double_t oy);
	void SetUnitTextColor(Color_t  color);
	void SetUnitTextFont(Font_t	 font);
	void SetUnitTextSize(Float_t size);

private:
	void UpdateVis();

	TVirtualPad *fCurrentPad; //! stores current pad we are drawn to

	Double_t   fUpdateInterval; // update interval
	TTimeStamp fLastUpdate;     // last time the view was painted

	TGNG     *fCurrentGNG; //! stores current gng provided OnFeedResult/OnPeekResult
	TGNGUnit *fCurrentBMU; //! stores current unit provided OnFeedResult/OnPeekResult

	Color_t	fEdgeColor; // color used for edges
	Style_t	fEdgeStyle; // style used for edges
	Width_t	fEdgeWidth; // width used for edges

	Color_t	fUnitColor; // color used for units
	Size_t	fUnitSize;  // size used for units
	Style_t	fUnitStyle;	// marker style used for units

	Color_t	fUnitMColor; // color used for units that are marked
	Size_t	fUnitMSize;  // size used for units that are marked
	Style_t	fUnitMStyle; // marker style used for units that are marked

	Double_t fUnitTextOffsetX; // unit text x offset from center
	Double_t fUnitTextOffsetY; // unit text y offset from center
	Color_t  fUnitTextColor;   // unit text color
	Font_t	 fUnitTextFont;    // unit text font
	Float_t	 fUnitTextSize;    // unit text size

};



#endif


